/*
; Example_05.
; ===========

;------------------------------------------------------------------------
; Author:	Franzi Edo.	The 03/04/98
; Modifs:
;
; Project:	KHEPERA
; Goal:		This PRGM launches 3 processes.
;			Process 0: "alive" it toggles the LED 0 every 500 ms.
;			Process 1: it displays the "tic-count" every 1 s.
;			Process 2: it synchronises the process on a character coming
;					   from the "SER" channel.
;					   If the character is '0', kill the process [0]. 
;					   If the character is '1', kill the process [1].
;					   If the character is '2', kill the process [2].
;					   Otherwise it toggles the LED 1. 
;
; This module is (c) of Edoardo Franzi 1992-1998
; franzi@k-team.com
;
;------------------------------------------------------------------------
*/

#include	<khesys.h>
#include 	<stdlib.h>
#include 	<stdio.h>

static int32	vIDProcess[3];

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

static void process_0()
	{

	for(;;)
		{
		tim_suspend_task(500);
		var_change_led(0);
		}
	}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process displays the "tic-count" value
 * every 1 s.
 *
 */

static void process_1()
	{
	uint32	ticcount;

	for(;;)
		{
		tim_suspend_task(1000);
		ticcount = tim_get_ticcount();
		RESERVE_COM
		printf("Tic-count value = %d\n", ticcount);
		RELEASE_COM
		}
	}

/*
 * Process 2
 * ^^^^^^^^^
 *
 * This process waits for a character coming
 * from the "SER" channel.
 * Possibe actions:
 * If the character is '0', kill the process [0]. 
 * If the character is '1', kill the process [1].
 * If the character is '2', kill the process [2].
 * Otherwise it toggles the LED 1. 
 *
 */
 
static void process_2()
	{
	int32	status;
	uint8 	strout[4];
	int32	aChar;

/* Flush the channel */

	RESERVE_COM
	while ((com_get_status_channel() & 0x2) == 0x2) com_receive_byte();
	RELEASE_COM

	for(;;)
		{
	
/* Syc. the process with a in character coming from the "SER" channel */

		tim_wait_sync(0x1 << 5);
		RESERVE_COM

/* Echo ... and action */

		aChar = com_receive_byte();
		if (aChar > 0)
			{
			var_change_led(1);
			strout[0] = (uint8)aChar;
			com_send_buffer(strout, 1);
			switch ((char)aChar)
				{
				case '0':
					{
					status = kill_task(vIDProcess[0]);
					if (status < 0)
						{
						RELEASE_COM
						exit(0);
						}
					break;
					}
				case '1':
					{
					status = kill_task(vIDProcess[1]);
					if (status < 0)
						{
						RELEASE_COM
						exit(0);
						}

					break;
					}

/* Before to kill myself (process 2), get free the channel */

				case '2':
					{
					RELEASE_COM
					status = kill_task(vIDProcess[2]);
					exit(0);
					}
				default: break;
				}
			}
		RELEASE_COM
		}
	}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

void main(void)
	{
	int32	status;

	/*
	--------------------------|------------------------------------------------|
	*/

	static char	prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
	static char	prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";
	static char	prName_2[] = "User 2 process, EF-98                            Rev. 1.00\r\n";
	
	com_reset();
	var_reset();

	status = install_task(prName_0, 800, process_0);
	if (status == -1) exit(0);
	vIDProcess[0] = (uint32)status;

	status = install_task(prName_1, 800, process_1);
	if (status == -1) exit(0);
	vIDProcess[1] = (uint32)status;

	status = install_task(prName_2, 800, process_2);
	if (status == -1) exit(0);
	vIDProcess[2] = (uint32)status;
	}
