/*
; Example_07.
; ===========

;------------------------------------------------------------------------
; Author:	Franzi Edo.	The 03/04/98
; Modifs:
;
; Project:	KHEPERA
; Goal:		This PRGM launches 3 processes.
;			Process 0: "alive" it toggles the LED 0 every 500 ms.
;			Process 1: it generates an event every 1 s.
;			Process 2: it waits for an event coming from the process 1
;					   and it displays the active processes.
;
; This module is (c) of Edoardo Franzi 1992-1998
; franzi@k-team.com
;
;------------------------------------------------------------------------
*/

#include	<khesys.h>
#include 	<stdlib.h>
#include 	<stdio.h>

static int32	vIDProcess[3];

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

static void process_0()
	{

	for(;;)
		{
		tim_suspend_task(500);
		var_change_led(0);
		}
	}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process generates an event
 * every 1 s.
 *
 */

static void process_1()
	{

	for(;;)
		{
		tim_suspend_task(1000);
		RESERVE_COM
		printf("\nPhysical process ID. %d generates an event\n", tim_get_id());
		RELEASE_COM
		tim_generate_event();
		}
	}

/*
 * Process 2
 * ^^^^^^^^^
 *
 * This process waits for an event coming from the process 1.
 * Then it displays the list of the installed processes.
 *
 */

static void process_2()
	{
	PROCDESC	*descProcess;
	int			i;

	for(;;)
		{
		tim_wait_event(vIDProcess[1]);
		RESERVE_COM
		printf("\nPhysical process ID. %d has received an event\n", tim_get_id());
		RELEASE_COM
		for(i = 0; i < 31; i++)
			{
			descProcess = tim_get_task_des_ptr();
			descProcess = descProcess + i;
			switch (descProcess->oStaProc)
				{
				case (1 << bPres):
					{
					RESERVE_COM
					printf("%s", descProcess->oPtrTextID);
					RELEASE_COM
					break;
					}
				default: break;
				}
			}
		}
	}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

void main(void)
	{
	int32	status;

	/*
	--------------------------|------------------------------------------------|
	*/

	static char	prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
	static char	prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";
	static char	prName_2[] = "User 2 process, EF-98                            Rev. 1.00\r\n";
	
	com_reset();
	var_reset();

	status = install_task(prName_0, 800, process_0);
	if (status == -1) exit(0);
	vIDProcess[0] = (uint32)status;

	status = install_task(prName_1, 800, process_1);
	if (status == -1) exit(0);
	vIDProcess[1] = (uint32)status;

	status = install_task(prName_2, 800, process_2);
	if (status == -1) exit(0);
	vIDProcess[2] = (uint32)status;
	}
