/*
; test_gripper.
; =============

;------------------------------------------------------------------------
; Author:	Franzi Edo.	The 03/04/98
; Modifs:
;
; Project:	KHEPERA
; Goal:		Test of the gripper driver.
;
; This module is (c) of Edoardo Franzi 1992-1998
; franzi@k-team.com
;
;------------------------------------------------------------------------
*/

#include	<khesys.h>
#include	<gripper.h>
#include 	<stdlib.h>
#include 	<stdio.h>

uint32	vIDProcess[2];

/* Function 1 */
/* >>>>>>>>>> */

void function_1()
	{
	int32	status;
	uint8	vVersion, vRevision;

	status = gripper_getVersion();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	else
		{
		vVersion = (uint8)(status >> 8);
		vRevision = (uint8)(status);
		RESERVE_COM
		printf("Version %d.%d\n", vVersion, vRevision);
		RELEASE_COM
		}
	}

/* Function 2 */
/* >>>>>>>>>> */

void function_2()
	{
	int32	status;
	uint8	elevation;
	
	elevation = (uint8)tim_get_ticcount();
	status = gripper_putArmPosition(elevation);
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	}

/* Function 3 */
/* >>>>>>>>>> */

void function_3()
	{
	int32	status;
	
	status = gripper_openGripper();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	}

/* Function 4 */
/* >>>>>>>>>> */

void function_4()
	{
	int32	status;
	
	status = gripper_closeGripper();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	}

/* Function 5 */
/* >>>>>>>>>> */

void function_5()
	{
	int32	status;
	
	status = gripper_getGripperPosition();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	else
		{
		RESERVE_COM
		printf("Position %d\n", (uint8)status);
		RELEASE_COM
		}
	}

/* Function 6 */
/* >>>>>>>>>> */

void function_6()
	{
	int32	status;
	
	status = gripper_getArmPosition();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	else
		{
		RESERVE_COM
		printf("Position %d\n", (uint8)status);
		RELEASE_COM
		}
	}

/* Function 7 */
/* >>>>>>>>>> */

void function_7()
	{
	int32	status;
	
	status = gripper_getObjectStatus();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	else
		{
		RESERVE_COM
		printf("Object %d\n", (uint8)status);
		RELEASE_COM
		}
	}

/* Function 8 */
/* >>>>>>>>>> */

void function_8()
	{
	int32	status;
	
	status = gripper_getObjectResistivity();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	else
		{
		RESERVE_COM
		printf("Resistivity %d\n", (uint8)status);
		RELEASE_COM
		}
	}

/* Function 9 */
/* >>>>>>>>>> */

void function_9()
	{
	int32	status;
	
	status = gripper_getJumperState();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	else
		{
		RESERVE_COM
		printf("Jumper %d\n", (uint8)status);
		RELEASE_COM
		}
	}

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */
 
void process_0()
	{

	for(;;)
		{
		tim_suspend_task(500);
		var_change_led(0);
		}
	}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * Launch a function.
 *
 */

void process_1()
	{
	
	for(;;)
		{
		tim_suspend_task(500);
		function_1();
		}
	}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

void main(void)
	{
	int32	status;

	/*
	--------------------------|------------------------------------------------|
	*/

	static char	prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
	static char	prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";

	var_reset();
	gripper_reset();

	status = install_task(prName_0, 800, process_0);
	if (status == -1) exit(0);
	vIDProcess[0] = (uint32)status;

	status = install_task(prName_1, 800, process_1);
	if (status == -1) exit(0);
	vIDProcess[1] = (uint32)status;
	}
