/*
 *	STDLIB.H	additional definitions
 */

#ifndef _STDLIB_H
#define _STDLIB_H

#include <types.h>
#include <stddef.h>

#define EXIT_SUCCESS	(0)
#define EXIT_FAILURE	(-1)

#define RAND_MAX	(0x7FFF)	/* max. value from rand() */

typedef	struct	{	/* type returned by ldiv */
	long	quot;
	long	rem;
	}	ldiv_t;
typedef	struct	{	/* type returned by div */
	int	quot;
	int	rem;
	}	div_t;

extern	char	*optarg;
extern	int	opterr;
extern	int	optind;

extern	void	abort	(void);
extern	int	abs	(int value);
extern	void	*alloca	(size_t size);
extern	double	atof	(const char *s);
extern	int	atoi	(const char *s);
extern	long	atol	(const char *s);
extern	void	*bsearch(const void *key, const void *base, size_t nelem,
			 size_t width, int (*fcmp)(const void*,const void*));
extern	void	*calloc	(size_t nitems, size_t size);
extern	div_t	div	(int numer, int denom);
extern	void	exit	(int status);
extern	void	_exit	(int status);
extern	int	ffs	(int value);
extern	void	free	(void *block);
extern	char	*getenv (const char *name);
extern	char	*itoa	(int value, char *string, int radix);
extern	long	labs	(long value);
extern	ldiv_t	ldiv	(long numer, long denom);
extern	char	*ltoa	(long value, char *string, int radix);
extern	void	*malloc (size_t size);
extern	long	memavail(void);
extern	long	msize	(void *block);
extern	int	putenv	(const char *name);
extern	void	qsort	(void *base, size_t nelem, size_t width,
			int (*fcmp)(const void*,const void*));
extern	int	rand	(void);		/* 0..RAND_MAX */
extern	long	random	(void);		/* 0..(2**32)-1 */
extern	void	randomize(void);	/* for rand() */
extern	void	*realloc(void *block, size_t size);
extern	void	srand	(unsigned seed);
extern	void	srandom	(unsigned long seed);
extern	double	strtod	(const char *s, char **endptr);
extern	long	strtol	(const char *s, char **endptr, int radix);
extern	unsigned long strtoul (const char *s, char **endptr, int radix);
extern	char	*ultoa	(unsigned long value, char *string, int radix);

#define max(a,b)        (((a) > (b)) ? (a) : (b))
#define min(a,b)        (((a) < (b)) ? (a) : (b))

#endif	/* _STDLIB_H */

